// Solutions to the Advent Of Code 2024.
// Copyright (C) 2024  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#include <catch2/catch_test_macros.hpp>

#include <aoc/Solver.hpp>
#include <aoc/SolverEngine.hpp>
#include <aoc/HistorianHysteria.hpp>

#include <aocTests/TestContext.hpp>

#define REQUIRE_MESSAGE(cond, msg) if (!(cond)) { INFO(msg); REQUIRE(cond); }

class TestCaseBase
{
public:
    TestContext fullDataContext{ getInputPaths() };
    TestContext exampleDataContext{ getExampleInputPaths() };
    void run(const std::unique_ptr<Solver>&& solver, const long long int expected1,
        const long long int expected2, const TestContext& context)
    {
        SolverEngine solverEngine{ context.getInputPaths() };
        solverEngine.run(*solver);

        REQUIRE(expected1 == solver->getResultPart1());
        REQUIRE(expected2 == solver->getResultPart2());
    }
private:
    std::vector<std::string> getInputPaths() const
    {
        return std::vector<std::string>{ "data", "../../../data", "../../../../data" };
    }
    std::vector<std::string> getExampleInputPaths() const
    {
        return std::vector<std::string>{ "data/examples", "../../../data/examples",
            "../../../../data/examples" };
    }
};

TEST_CASE("[HistorianHysteriaTests]")
{
    TestCaseBase test;
    SECTION("FullData")
    {
        test.run(std::make_unique<HistorianHysteria>(), 2176849, 23384288, test.fullDataContext);
    }
    SECTION("ExampleData")
    {
        test.run(std::make_unique<HistorianHysteria>(), 11, 31, test.exampleDataContext);
    }
}
